searchNodes=[{"doc":"Elixir module to help you write matchspecs. contains functions which transform elixir-style functions into erlang matchspecs, and vice versa. Functions to matchspecs For transforming elixir-style functions into matchspecs, the following restrictions apply: Function form The function must use the Kernel.fn/1 macro as its form, or use defmatchspec/2 or defmatchspecp/2 , where the matchspecs form is similar to the Kernel.fn/1 form The function must have arity 1. Function argument matching The function may only match whole variables or tuple patterns. Only one tuple pattern may be matched. if your tuple contains a binary pattern match the binary pattern may only consist of bytes and strings, and only the size/1 modifier is allowed. bitstrings matching is not supported conversions such as float are not supported. Guards and return expression The function may only use guards in its when section. for defmatchspec/2 , defmatchspecp/2 , or fun2msfun/4 : a Kernel.in/2 guard may take a bound variable as its second parameter. The function may only return a single expression that (optionally) uses guard functions to transform matches. Examples (allowed) Binding a top-level variable fn tuple -&gt; tuple end Top-level tuple match fn { _ , _ , value } -&gt; value end When with guard fn { _ , _ , value } when is_integer ( value ) -&gt; value end Limitations on local guards local ( defguardp ) guards are only supported when the macro is inside of a function body, due to limitations on macro resolution timing. The following use cases are currently NOT supported: fun2ms/2 outside of a function body (inside the module body) fun2msfun/4 outside of function body ( :lambda , :def , or :defp ) The following use cases are currently supported: fun2ms/2 inside a function body fun2msfun/4 inside of a function body ( :lambda only) defmatchspec/2 always defmatchspecp/2 always Structure matching inside a tuple fn { %{ key : a } } -&gt; a end Binary matching inside a tuple fn { &lt;&lt; &quot;foo&quot; :: binary , 42 , a :: binary &gt;&gt; } -&gt; a end Result expression modification by guards fn { a , b } -&gt; a + b end Examples (disallowed) Arity not 1 fn foo , bar -&gt; foo + bar end Multiple tuple matches fn { _ , :foo } = { :bar , value } -&gt; value end Non-tuple top level match fn &quot;foo&quot; &lt;&gt; bar -&gt; bar end fn %{ foo : bar } -&gt; bar end Disallowed type in binary match fn { &lt;&lt; foo :: integer - big - endian &gt;&gt; } -&gt; foo end Non-guard function in match fn { a } when String . starts_with? ( a , &quot;foo&quot; ) -&gt; a end Non-guard function in result fn { a , b } -&gt; a ++ b end Note The restrictions on binary matching exist due to limitations on the BIFs available to ets and may change in the future if OTP comes to support these conversions in its kernel.","ref":"MatchSpec.html","title":"MatchSpec","type":"module"},{"doc":"Writes a matchspec-generating function based on a body. You may provide multiple function bodies. This macro uses the same backend as fun2msfun/4 and will generate identical code to that macro. Example: use MatchSpec defmatchspec my_matchspec ( value ) do { key , ^ value } when key !== :foo -&gt; key end This generates the equivalent to the following function: def my_matchspec ( value ) do [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$2&quot; , { :const , value } } , { :&quot;=/=&quot; , :&quot;$1&quot; , { :const , :foo } } ] , [ :&quot;$1&quot; ] ] end","ref":"MatchSpec.html#defmatchspec/2","title":"MatchSpec.defmatchspec/2","type":"macro"},{"doc":"Writes a matchspec-generating function based on a body. You may provide multiple function bodies. This macro uses the same backend as fun2msfun/4 and will generate identical code. see defmatchspec/2 for details","ref":"MatchSpec.html#defmatchspecp/2","title":"MatchSpec.defmatchspecp/2","type":"macro"},{"doc":"converts a function ast into an ets matchspec. The function must also be &quot; fn ast&quot;; you can't pass a shorthand lambda or a lambda to an existing lambda. The function lambda form is only used as a scaffolding to represent ets matching and filtering operations, by default it will not be instantiated into bytecode of the resulting module. iex&gt; require MatchSpec iex&gt; MatchSpec . fun2ms ( fn tuple = { k , v } when v &gt; 1 and v &lt; 10 -&gt; tuple end ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :andalso , { :&gt; , :&quot;$2&quot; , 1 } , { :&lt; , :&quot;$2&quot; , 10 } } ] , [ :&quot;$_&quot; ] } ] You can use variables from the calling scope in the filters. iex&gt; require MatchSpec iex&gt; my_atom = :foo iex&gt; MatchSpec . fun2ms ( fn tuple = { k , _ } when k === my_atom -&gt; tuple end ) [ { { :&quot;$1&quot; , :_ } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :foo } } ] , [ :&quot;$_&quot; ] } ] If you would also like the equivalent lambda, pass with_fun: true as an option and the fun2ms/2 macro will emit a tuple of the matchspec and the lambda. iex&gt; { ms , fun } = MatchSpec . fun2ms ( fn { :key , value } -&gt; value end , with_fun : true ) iex&gt; :ets . test_ms ( { :key , &quot;value&quot; } , ms ) { :ok , &quot;value&quot; } iex&gt; fun . ( { :key , &quot;value&quot; } ) &quot;value&quot; This macro uses the same backend as fun2msfun/4 and will emit the same matchspec as if you passed no parameters to fun2msfun/4","ref":"MatchSpec.html#fun2ms/2","title":"MatchSpec.fun2ms/2","type":"macro"},{"doc":"converts a function into a function that generates a match spec based on bindings. This can be used to either create a named function or an anonymous function. If you would like to use one of the free variables in your function as a part of the head of the match, you must pin it. if you omit the first parameter, it will create an anonymous function. Basic example with :lambda (default): iex&gt; require MatchSpec # using a variable in the match iex&gt; lambda = MatchSpec . fun2msfun ( :lambda , fn { key , value } when key === target -&gt; value end , [ target ] ) iex&gt; lambda . ( :key ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :key } } ] , [ :&quot;$2&quot; ] } ] #pinning a variable iex&gt; lambda2 = MatchSpec . fun2msfun ( fn { ^ key , value } -&gt; value end , [ key ] ) iex&gt; lambda2 . ( :key ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :key } } ] , [ :&quot;$2&quot; ] } ] Note that the bindings parameter acts like pattern matching on function arguments: They may use complex matches and there can be more than one, the arity of the anonymous (or def/defp) function matches the length of the bindings argument. iex&gt; require MatchSpec iex&gt; lambda = MatchSpec . fun2msfun ( :lambda , fn { ^ key , ^ value } -&gt; true end , [ %{ key : key } , value ] ) iex&gt; lambda . ( %{ key : :key } , :value ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :key } } , { :&quot;=:=&quot; , :&quot;$2&quot; , { :const , :value } } ] , [ true ] } ] Example with ( :def / :defp ): require MatchSpec MatchSpec . fun2msfun ( :def , :matchspec , fn { key , value } when key == target -&gt; value end , [ target ] )","ref":"MatchSpec.html#fun2msfun/4","title":"MatchSpec.fun2msfun/4","type":"macro"},{"doc":"converts a matchspec into elixir AST for functions. Unfortunately, the ast generator cannot guess names for variables, so variable names are set by the numerical value of the matchspec token The second parameter takes two modes: :ast emits elixir ast to write a lambda. iex&gt; MatchSpec . ms2fun ( [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ ] , [ :&quot;$2&quot; ] } ] , :ast ) { :fn , [ ] , [ { :-&gt; , [ ] , [ [ { :{} , [ ] , [ { :v1 , [ ] , nil } , { :v2 , [ ] , nil } ] } ] , { :v2 , [ ] , nil } ] } ] } :code outputs formatted elixir code. iex&gt; MatchSpec . ms2fun ( [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ ] , [ :&quot;$2&quot; ] } , { { :&quot;$1&quot; } , [ ] , [ :&quot;$_&quot; ] } ] , :code ) &quot;&quot;&quot; fn {v1, v2} -&gt; v2 tuple = {v1} -&gt; tuple end &quot;&quot;&quot;","ref":"MatchSpec.html#ms2fun/2","title":"MatchSpec.ms2fun/2","type":"function"},{"doc":":ets matchspec helper library for elixir. Exposes the fun2ms/1 macro which transforms elixir-style function code into ets matchspecs. iex&gt; require MatchSpec iex&gt; MatchSpec . fun2ms ( fn { key , value } when key === :foo -&gt; value end ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :foo } } ] , [ :&quot;$2&quot; ] } ] Also exposes the ms2fun/2 function which converts a matchspec to function code or ast which represents a function that performs the same task as the ets matchspec iex&gt; MatchSpec . ms2fun ( [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :foo } } ] , [ :&quot;$2&quot; ] } ] , :code ) &quot;fn {v1, v2} when v1 === :foo -&gt; v2 end&quot; iex&gt; :ets . test_ms ( { :foo , :bar } , MatchSpec . fun2ms ( fn { key , value } when key === :foo -&gt; value end ) ) { :ok , :bar } Provides fun2msfun/4 macro which can be used to parametrize the matchspec: iex&gt; require MatchSpec iex&gt; lambda = MatchSpec . fun2msfun ( fn { ^ key , value } -&gt; value end , [ key ] ) iex&gt; lambda . ( :key ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :key } } ] , [ :&quot;$2&quot; ] } ] Provides defmatchspec/2 and defmatchspecp/2 macros which can be used to directly generate functions in your module defmodule MyModule do use MatchSpec defmatchspec my_matchspec ( key ) do { ^ key , value } -&gt; value end end MyModule . my_matchspec ( :key ) [ { { :&quot;$1&quot; , :&quot;$2&quot; } , [ { :&quot;=:=&quot; , :&quot;$1&quot; , { :const , :key } } ] , [ :&quot;$2&quot; ] } ]","ref":"readme.html","title":"MatchSpec","type":"extras"},{"doc":"The package can be installed by adding match_spec to your list of dependencies in mix.exs : def deps do [ { :match_spec , &quot;~&gt; 0.3.1&quot; } ] end Documentation can be found at https://hexdocs.pm/match_spec .","ref":"readme.html#installation","title":"MatchSpec - Installation","type":"extras"}]